@@ -1,12 +1,10 @@
-"""
-Module for the Executable base class.
-"""
+"""Module for the Executable base class."""
 
 import os
-from pathlib import Path
 import string
 import sys
-import sysconfig
+from pathlib import Path
+from sysconfig import get_config_var, get_platform
 from typing import Optional, Union
 
 from .common import get_resource_file_path, validate_args
@@ -15,6 +13,7 @@ from .exception import ConfigError
 STRINGREPLACE = list(
     string.whitespace + string.punctuation.replace(".", "").replace("_", "")
 )
+WIN32 = sys.platform == "win32"
 
 __all__ = ["Executable"]
 
@@ -33,8 +32,10 @@ class Executable:
         icon: Optional[Union[str, Path]] = None,
         shortcut_name: Optional[str] = None,
         shortcut_dir: Optional[Union[str, Path]] = None,
-        copyright: Optional[str] = None,
+        copyright: Optional[str] = None,  # pylint: disable=W0622
         trademarks: Optional[str] = None,
+        manifest: Optional[Union[str, Path]] = None,
+        uac_admin: bool = False,
         *,
         initScript: Optional[str] = None,
         targetName: Optional[str] = None,
@@ -58,6 +59,8 @@ class Executable:
         )
         self.copyright = copyright
         self.trademarks = trademarks
+        self.manifest = manifest
+        self.uac_admin = uac_admin
 
     def __repr__(self):
         return f"<Executable script={self.main_script}>"
@@ -75,16 +78,19 @@ class Executable:
     @base.setter
     def base(self, name: Optional[Union[str, Path]]):
         name = name or "Console"
-        py_version_nodot = sysconfig.get_config_var("py_version_nodot")
-        platform_nodot = sysconfig.get_platform().replace(".", "")
-        name_plat = f"{name}-cp{py_version_nodot}-{platform_nodot}"
-        exe_extension = ".exe" if sys.platform == "win32" else ""
-        self._base: Path = get_resource_file_path(
-            "bases", name_plat, exe_extension
-        )
+        if WIN32:
+            py_version_nodot = get_config_var("py_version_nodot")
+            platform_nodot = get_platform().replace(".", "").replace("-", "_")
+            soabi = f"cp{py_version_nodot}-{platform_nodot}"
+            suffix = ".exe"
+        else:
+            soabi = get_config_var("SOABI")
+            suffix = ""
+        name_base = f"{name}-{soabi}"
+        self._base: Path = get_resource_file_path("bases", name_base, suffix)
         if self._base is None:
-            raise ConfigError(f"no base named {name!r}")
-        self._ext: str = exe_extension
+            raise ConfigError(f"no base named {name!r} ({name_base!r})")
+        self._ext: str = suffix
 
     @property
     def icon(self) -> str:
@@ -153,6 +159,25 @@ class Executable:
     def main_script(self, name: Union[str, Path]):
         self._main_script: Path = Path(name)
 
+    @property
+    def manifest(self) -> Optional[str]:
+        """
+        :return: the XML schema of the manifest which is to be included in the
+        frozen executable
+        :rtype: str
+
+        """
+        return self._manifest
+
+    @manifest.setter
+    def manifest(self, name: Optional[Union[str, Path]]) -> None:
+        self._manifest: Optional[str] = None
+        if name is None:
+            return
+        if isinstance(name, str):
+            name = Path(name)
+        self._manifest = name.read_text(encoding="utf-8")
+
     @property
     def shortcut_name(self) -> str:
         """
