@@ -1,23 +1,29 @@
-"""
-Base class for finding modules.
-"""
+"""Module Finder - discovers what modules are required by the code."""
 
-import dis
-from importlib.abc import ExecutionLoader
 import importlib.machinery
 import logging
-import os
 import sys
+from contextlib import suppress
+from importlib.abc import ExecutionLoader
+from pathlib import Path, PurePath
+from sysconfig import get_config_var
 from types import CodeType
-from typing import Dict, List, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Sequence, Set, Tuple, Union
+
 import opcode
 
-from cx_Freeze.common import code_object_replace
-from cx_Freeze.module import Module
-
+from ._compat import cached_property
+from .common import (
+    IncludesList,
+    InternalIncludesList,
+    code_object_replace,
+    get_resource_file_path,
+    process_path_specs,
+)
+from .module import ConstantsModule, Module
 
 BUILD_LIST = opcode.opmap["BUILD_LIST"]
-INPLACE_ADD = opcode.opmap["INPLACE_ADD"]
+EXTENDED_ARG = opcode.opmap["EXTENDED_ARG"]
 LOAD_CONST = opcode.opmap["LOAD_CONST"]
 IMPORT_NAME = opcode.opmap["IMPORT_NAME"]
 IMPORT_FROM = opcode.opmap["IMPORT_FROM"]
@@ -26,6 +32,8 @@ STORE_FAST = opcode.opmap["STORE_FAST"]
 STORE_NAME = opcode.opmap["STORE_NAME"]
 STORE_GLOBAL = opcode.opmap["STORE_GLOBAL"]
 STORE_OPS = (STORE_NAME, STORE_GLOBAL)
+HAVE_ARGUMENT = opcode.HAVE_ARGUMENT
+
 
 DeferredList = List[Tuple[Module, Module, List[str]]]
 
@@ -33,35 +41,40 @@ __all__ = ["Module", "ModuleFinder"]
 
 
 class ModuleFinder:
+    """ModuleFinder base class."""
+
     def __init__(
         self,
-        include_files: Optional[List[str]] = None,
+        include_files: Optional[IncludesList] = None,
         excludes: Optional[List[str]] = None,
-        path: Optional[str] = None,
-        replace_paths: Optional[List[str]] = None,
+        path: Optional[List[str]] = None,
+        replace_paths: Optional[List[Tuple[str, str]]] = None,
         zip_include_all_packages: bool = False,
         zip_exclude_packages: Optional[List[str]] = None,
         zip_include_packages: Optional[List[str]] = None,
-        constants_module=None,
-        zip_includes: Optional[List[str]] = None,
+        constants_module: Optional[ConstantsModule] = None,
+        zip_includes: Optional[IncludesList] = None,
     ):
-        self.include_files = include_files or []
-        self.excludes = dict.fromkeys(excludes or [])
+        self.included_files: InternalIncludesList = process_path_specs(
+            include_files
+        )
+        self.excludes: Dict[str, Any] = dict.fromkeys(excludes or [])
         self.optimize_flag = 0
-        self.path = path or sys.path
+        self.path: List[str] = self._set_path(path)
         self.replace_paths = replace_paths or []
         self.zip_include_all_packages = zip_include_all_packages
         self.zip_exclude_packages = zip_exclude_packages or []
         self.zip_include_packages = zip_include_packages or []
         self.constants_module = constants_module
-        self.zip_includes = zip_includes or []
+        self.zip_includes: InternalIncludesList = process_path_specs(
+            zip_includes
+        )
         self.modules = []
         self.aliases = {}
-        self.exclude_dependent_files = {}
-        self._modules = dict.fromkeys(
+        self.excluded_dependent_files: Set[Path] = set()
+        self._modules: Dict[str, Optional[Module]] = dict.fromkeys(
             excludes or []
-        )  # type: Dict[str, Optional[Module]]
-        self._builtin_modules = dict.fromkeys(sys.builtin_module_names)
+        )
         self._bad_modules = {}
         self._hooks = __import__("cx_Freeze", fromlist=["hooks"]).hooks
         self._hooks.initialize(self)
@@ -77,28 +90,29 @@ class ModuleFinder:
         When cx_Freeze is built, these modules (and modules they load) are
         included in the startup zip file.
         """
-        self.IncludeModule("traceback")
-        self.IncludeModule("warnings")
-        self.IncludeModule("unicodedata")
-        self.IncludePackage("encodings")
-        self.IncludeModule("io")
-        self.IncludeModule("os")
-        self.IncludeModule("sys")
-        self.IncludeModule("zlib")
-        self.IncludeModule("collections.abc")
-        self.IncludeModule("importlib.abc")
+        self.include_module("traceback")
+        self.include_module("warnings")
+        self.include_module("unicodedata")
+        self.include_package("encodings")
+        self.include_module("io")
+        self.include_module("os")
+        self.include_module("sys")
+        self.include_module("zlib")
+        self.include_module("collections.abc")
+        self.include_module("importlib.abc")
 
     def _add_module(
         self,
         name: str,
-        path: Optional[List[str]] = None,
-        file_name: Optional[str] = None,
+        path: Optional[Sequence[Union[Path, str]]] = None,
+        file_name: Optional[Union[Path, str]] = None,
         parent: Optional[Module] = None,
     ) -> Module:
         """
         Add a module to the list of modules but if one is already found,
         then return it instead; this is done so that packages can be
-        handled properly."""
+        handled properly.
+        """
         module = self._modules.get(name)
         if module is None:
             module = Module(name, path, file_name, parent)
@@ -114,9 +128,9 @@ class ModuleFinder:
                 and module.name not in self.zip_exclude_packages
                 or module.name in self.zip_include_packages
             ):
-                module.store_in_file_system = False
+                module.in_file_system = 0
         if module.path is None and path is not None:
-            module.path = path
+            module.path = [Path(p) for p in path]
         if module.file is None and file_name is not None:
             module.file = file_name
         return module
@@ -129,6 +143,17 @@ class ModuleFinder:
             return self._get_parent_by_name(caller.name)
         return None
 
+    @cached_property
+    def _builtin_modules(self) -> Set[str]:
+        """The built-in modules are determined based on the cx_Freeze build."""
+        builtin_modules: Set[str] = set(sys.builtin_module_names)
+        dynload = get_resource_file_path("bases", "lib-dynload", "")
+        if dynload and dynload.is_dir():
+            # discard modules that exist in bases/lib-dynload
+            for file in dynload.iterdir():
+                builtin_modules.discard(file.name.partition(".")[0])
+        return builtin_modules
+
     def _ensure_from_list(
         self,
         caller: Module,
@@ -169,29 +194,23 @@ class ModuleFinder:
         suffixes = importlib.machinery.all_suffixes()
 
         for path in module.path:
-            try:
-                filenames = os.listdir(path)
-            except OSError:
-                continue
-
-            for filename in filenames:
-                fullname = os.path.join(path, filename)
-                if os.path.isdir(fullname):
-                    init_file = os.path.join(fullname, "__init__.py")
-                    if not os.path.exists(init_file):
+            for fullname in path.iterdir():
+                if fullname.is_dir():
+                    if not fullname.joinpath("__init__.py").exists():
                         continue
-                    name = filename
+                    name = fullname.name
                 else:
                     # We need to run through these in order to correctly pick
-                    # up PEP 3149 library names (e.g. .cpython-32mu.so).
+                    # up PEP 3149 library names
+                    # (e.g. .cpython-39-x86_64-linux-gnu.so).
                     for suffix in suffixes:
-                        if filename.endswith(suffix):
-                            name = filename[: -len(suffix)]
+                        if fullname.name.endswith(suffix):
+                            name = fullname.name[: -len(suffix)]
 
                             # Skip modules whose names appear to contain '.',
                             # as we may be using the wrong suffix, and even if
-                            # we're not, such module names will break the import
-                            # code.
+                            # we're not, such module names will break the
+                            # import code.
                             if "." not in name:
                                 break
 
@@ -252,8 +271,8 @@ class ModuleFinder:
             module = self._internal_import_module(name, deferred_imports)
 
         # old style relative import (regular 'import foo' in Python 2)
-        # the name given is tried in the current package, and if
-        # no match is found, sys.path is searched for a top-level module/pockage
+        # the name given is tried in the current package, and if no match
+        # is found, sys.path is searched for a top-level module/pockage
         elif relative_import_index < 0:
             parent = self._determine_parent(caller)
             if parent is not None:
@@ -298,16 +317,15 @@ class ModuleFinder:
 
     def _internal_import_module(
         self, name: str, deferred_imports: DeferredList
-    ):
+    ) -> Optional[Module]:
         """
         Internal method used for importing a module which assumes that the
         name given is an absolute name. None is returned if the module
-        cannot be found."""
-        try:
+        cannot be found.
+        """
+        with suppress(KeyError):
             # Check in module cache before trying to import it again.
             return self._modules[name]
-        except KeyError:
-            pass
 
         if name in self._builtin_modules:
             module = self._add_module(name)
@@ -328,6 +346,10 @@ class ModuleFinder:
             if parent_module is None:
                 return None
             path = parent_module.path
+            if path is None:
+                path = self.path
+            else:
+                path = [str(p) for p in path]
 
         if name in self.aliases:
             actual_name = self.aliases[name]
@@ -350,58 +372,72 @@ class ModuleFinder:
     def _load_module(
         self,
         name: str,
-        path: Union[str, List[str]],
+        path: Optional[Sequence[str]],
         deferred_imports: DeferredList,
         parent: Optional[Module] = None,
     ) -> Optional[Module]:
         """Load the module, searching the module spec."""
-        spec: Optional[importlib.machinery.ModuleSpec]
-        loader: ExecutionLoader
-        module: Module
+        spec: Optional[importlib.machinery.ModuleSpec] = None
+        loader: Optional[ExecutionLoader] = None
+        module: Optional[Module] = None
 
-        if isinstance(path, str):
-            # Include file as module
-            module = self._add_module(name, file_name=path, parent=parent)
-            ext = os.path.splitext(os.path.basename(path))[1]
-            if ext in importlib.machinery.SOURCE_SUFFIXES + [""]:
-                loader = importlib.machinery.SourceFileLoader(name, path)
-            elif ext in importlib.machinery.BYTECODE_SUFFIXES:
-                loader = importlib.machinery.SourcelessFileLoader(name, path)
-            elif ext in importlib.machinery.EXTENSION_SUFFIXES:
-                loader = importlib.machinery.ExtensionFileLoader(name, path)
-            else:
-                loader = None
-        else:
-            # Find modules to load
-            try:
-                # It's recommended to clear the caches first.
-                importlib.machinery.PathFinder.invalidate_caches()
-                spec = importlib.machinery.PathFinder.find_spec(name, path)
-            except Exception:
-                spec = None
-            if spec is None:
-                return None
-            # Handle special cases
+        # Find modules to load
+        try:
+            # It's recommended to clear the caches first.
+            importlib.machinery.PathFinder.invalidate_caches()
+            spec = importlib.machinery.PathFinder.find_spec(name, path)
+        except KeyError:
+            if parent:
+                # some packages use a directory with vendored modules
+                # without an __init__.py and are not considered namespace
+                # packages, then simulate a subpackage
+                module = self._add_module(
+                    name,
+                    path=[Path(path[0], name.rpartition(".")[-1])],
+                    parent=parent,
+                )
+                logging.debug("Adding module [%s] [PACKAGE]", name)
+                module.file = Path(path[0], "__init__.py")
+                module.source_is_string = True
+
+        if spec:
             loader = spec.loader
+            # Ignore built-in importers
             if loader is importlib.machinery.BuiltinImporter:
                 return None
             if loader is importlib.machinery.FrozenImporter:
                 return None
-            # Load package or namespae package
+            # Load package or namespace package
             if spec.submodule_search_locations:
-                path_list = list(spec.submodule_search_locations)
-                module = self._add_module(name, path=path_list, parent=parent)
+                module = self._add_module(
+                    name,
+                    path=list(spec.submodule_search_locations),
+                    parent=parent,
+                )
                 if spec.origin in (None, "namespace"):
                     logging.debug("Adding module [%s] [NAMESPACE]", name)
-                    path = os.path.join(path_list[0], "__init__.py")
-                    module.code = compile("", path, "exec")
-                    return module
-                logging.debug("Adding module [%s] [PACKAGE]", name)
-                path = spec.origin  # path of __init__
-                module.file = path
+                    module.file = module.path[0] / "__init__.py"
+                    module.source_is_string = True
+                else:
+                    logging.debug("Adding module [%s] [PACKAGE]", name)
+                    module.file = spec.origin  # path of __init__.py
             else:
-                path = spec.origin
-                module = self._add_module(name, file_name=path, parent=parent)
+                module = self._add_module(
+                    name, file_name=spec.origin, parent=parent
+                )
+
+        if module is not None:
+            self._load_module_code(module, loader, deferred_imports)
+        return module
+
+    def _load_module_code(
+        self,
+        module: Module,
+        loader: Optional[ExecutionLoader],
+        deferred_imports: DeferredList,
+    ) -> Optional[Module]:
+        name = module.name
+        path = str(module.file)
 
         if isinstance(loader, importlib.machinery.SourceFileLoader):
             logging.debug("Adding module [%s] [SOURCE]", name)
@@ -413,7 +449,9 @@ class ModuleFinder:
                 )
             except SyntaxError:
                 logging.debug("Invalid syntax in [%s]", name)
-                raise ImportError(f"Invalid syntax in {path}", name=name)
+                raise ImportError(
+                    f"Invalid syntax in {path}", name=name
+                ) from None
         elif isinstance(loader, importlib.machinery.SourcelessFileLoader):
             logging.debug("Adding module [%s] [BYTECODE]", name)
             # Load Python bytecode
@@ -422,6 +460,8 @@ class ModuleFinder:
                 raise ImportError(f"Bad magic number in {path}", name=name)
         elif isinstance(loader, importlib.machinery.ExtensionFileLoader):
             logging.debug("Adding module [%s] [EXTENSION]", name)
+        elif module.source_is_string:
+            module.code = compile("", path, "exec")
         else:
             raise ImportError(f"Unknown module loader in {path}", name=name)
 
@@ -441,7 +481,27 @@ class ModuleFinder:
         module.in_import = False
         return module
 
-    def _replace_package_in_code(self, module: Module) -> CodeType:
+    def _load_module_from_file(
+        self, name: str, filename: Path, deferred_imports: DeferredList
+    ) -> Optional[Module]:
+        """Load the module from the filename."""
+        loader: Optional[ExecutionLoader] = None
+
+        ext = filename.suffix
+        path = str(filename)
+        if ext == "" or ext in importlib.machinery.SOURCE_SUFFIXES:
+            loader = importlib.machinery.SourceFileLoader(name, path)
+        elif ext in importlib.machinery.BYTECODE_SUFFIXES:
+            loader = importlib.machinery.SourcelessFileLoader(name, path)
+        elif ext in importlib.machinery.EXTENSION_SUFFIXES:
+            loader = importlib.machinery.ExtensionFileLoader(name, path)
+
+        module = self._add_module(name, file_name=filename)
+        self._load_module_code(module, loader, deferred_imports)
+        return module
+
+    @staticmethod
+    def _replace_package_in_code(module: Module) -> CodeType:
         """
         Replace the value of __package__ directly in the code,
         when the module is in a package and will be stored in library.zip.
@@ -451,7 +511,7 @@ class ModuleFinder:
         # and is not defined in the module, like 'six' do
         if (
             module.parent is None
-            or module.in_file_system
+            or module.in_file_system >= 1
             or "__package__" in module.global_names
             or code is None
         ):
@@ -488,43 +548,64 @@ class ModuleFinder:
         if code is None:
             code = module.code
         # Prepare the new filename.
-        new_filename = original_filename = os.path.normpath(code.co_filename)
+        original_filename = Path(code.co_filename)
         for search_value, replace_value in self.replace_paths:
             if search_value == "*":
                 if top_level_module.file is None:
                     continue
-                search_value = os.path.dirname(top_level_module.file)
                 if top_level_module.path:
-                    search_value = os.path.dirname(search_value)
-                if search_value:
-                    search_value = search_value + os.path.sep
-            if original_filename.startswith(search_value):
-                new_filename = (
-                    replace_value + original_filename[len(search_value) :]
-                )
+                    search_dir = top_level_module.file.parent.parent
+                else:
+                    search_dir = top_level_module.file.parent
+            else:
+                search_dir = Path(search_value)
+            with suppress(ValueError):
+                new_filename = original_filename.relative_to(search_dir)
+                new_filename = replace_value / new_filename
                 break
+        else:
+            new_filename = original_filename
 
         # Run on subordinate code objects from function & class definitions.
         consts = list(code.co_consts)
-        for i in range(len(consts)):
-            if isinstance(consts[i], type(code)):
+        for i, const in enumerate(consts):
+            if isinstance(const, type(code)):
                 consts[i] = self._replace_paths_in_code(
-                    top_level_module, consts[i]
+                    top_level_module, const
                 )
 
         return code_object_replace(
-            code, co_consts=consts, co_filename=new_filename
+            code, co_consts=consts, co_filename=str(new_filename)
         )
 
     def _run_hook(self, hook: str, module_name: str, *args) -> None:
-        """
-        Run hook (load or missing) for the given module if one is present.
-        """
-        name = "{}_{}".format(hook, module_name.replace(".", "_"))
-        method = getattr(self._hooks, name, None)
+        """Run hook (load or missing) for the given module if one is
+        present."""
+        normalized_name = module_name.replace(".", "_")
+        method = getattr(self._hooks, f"{hook}_{normalized_name}", None)
         if method is not None:
             method(self, *args)
 
+    @staticmethod
+    def _set_path(path: Optional[List[str]] = None) -> Set[str]:
+        """Set the path to search for modules, and fix the path for built-in
+        modules when it differs from the running python built-in modules."""
+        path = path or sys.path
+        if path == sys.path:
+            dynload = get_resource_file_path("bases", "lib-dynload", "")
+            if dynload and dynload.is_dir():
+                # add bases/lib-dynload to the finder path
+                insert_at = 0
+                dest_shared = get_config_var("DESTSHARED")
+                if dest_shared:
+                    for i, pth in enumerate(path):
+                        if pth == dest_shared:
+                            insert_at = i
+                            break
+                path = path.copy()
+                path.insert(insert_at, str(dynload))
+        return path
+
     def _scan_code(
         self,
         code,
@@ -532,25 +613,33 @@ class ModuleFinder:
         deferred_imports: DeferredList,
         top_level: bool = True,
     ):
-        """
-        Scan code, looking for imported modules and keeping track of the
+        """Scan code, looking for imported modules and keeping track of the
         constants that have been created in order to better tell which
-        modules are truly missing.
-        """
+        modules are truly missing."""
         arguments = []
         imported_module = None
-        for _index, op, arg in dis._unpack_opargs(code.co_code):
+        co_code = code.co_code
+        extended_arg = 0
+        for i in range(0, len(co_code), 2):
+            opc = co_code[i]
+            if opc >= HAVE_ARGUMENT:
+                arg = co_code[i + 1] | extended_arg
+                extended_arg = (arg << 8) if opc == EXTENDED_ARG else 0
+            else:
+                arg = None
+                extended_arg = 0
 
             # keep track of constants (these are used for importing)
             # immediately restart loop so arguments are retained
-            if op == LOAD_CONST:
+            if opc == LOAD_CONST:
                 arguments.append(code.co_consts[arg])
                 continue
 
             # import statement: attempt to import module
-            if op == IMPORT_NAME:
+            if opc == IMPORT_NAME:
                 name = code.co_names[arg]
                 if len(arguments) >= 2:
+                    # pylint: disable-next=W0632
                     relative_import_index, from_list = arguments[-2:]
                 else:
                     relative_import_index = -1
@@ -574,12 +663,14 @@ class ModuleFinder:
 
             # import * statement: copy all global names
             elif (
-                op == IMPORT_STAR and top_level and imported_module is not None
+                opc == IMPORT_STAR
+                and top_level
+                and imported_module is not None
             ):
                 module.global_names.update(imported_module.global_names)
 
             # store operation: track only top level
-            elif top_level and op in STORE_OPS:
+            elif top_level and opc in STORE_OPS:
                 name = code.co_names[arg]
                 module.global_names.add(name)
 
@@ -594,55 +685,76 @@ class ModuleFinder:
                     constant, module, deferred_imports, top_level=False
                 )
 
-    def AddAlias(self, name: str, alias_for: str) -> None:
-        """
-        Add an alias for a particular module; when an attempt is made to
+    def add_alias(self, name: str, alias_for: str) -> None:
+        """Add an alias for a particular module; when an attempt is made to
         import a module using the alias name, import the actual name instead.
         """
         self.aliases[name] = alias_for
 
-    def AddConstant(self, name: str, value: str) -> None:
+    def add_constant(self, name: str, value: str) -> None:
+        """Makes available a constant in the module BUILD_CONSTANTS which is
+        used in the initscripts."""
         self.constants_module.values[name] = value
 
-    def ExcludeDependentFiles(self, filename: str) -> None:
-        self.exclude_dependent_files[filename] = None
+    def exclude_dependent_files(self, filename: Union[Path, str]) -> None:
+        """Exclude the dependent files of the named file from the resulting
+        frozen executable."""
+        if not isinstance(filename, Path):
+            filename = Path(filename)
+        self.excluded_dependent_files.add(filename)
 
-    def ExcludeModule(self, name: str) -> None:
-        """Exclude the named module from the resulting frozen executable."""
-        self.excludes[name] = None
-        self._modules[name] = None
+    def exclude_module(self, name: str) -> None:
+        """Exclude the named module and its submodules from the resulting
+        frozen executable."""
+        modules_to_exclude = [name] + [
+            mod for mod in self._modules if mod.startswith(f"{name}.")
+        ]
+        for mod in modules_to_exclude:
+            self.excludes[mod] = None
+            self._modules[mod] = None
 
-    def IncludeFile(self, path: str, name: Optional[str] = None) -> Module:
+    def include_file_as_module(
+        self, path: Union[Path, str], name: Optional[str] = None
+    ) -> Module:
         """Include the named file as a module in the frozen executable."""
+        if isinstance(path, str):
+            path = Path(path)
         if name is None:
-            name = os.path.splitext(os.path.basename(path))[0]
+            name = path.name.partition(".")[0]
         deferred_imports: DeferredList = []
-        module = self._load_module(name, path, deferred_imports)
+        module = self._load_module_from_file(name, path, deferred_imports)
         self._import_deferred_imports(deferred_imports)
         return module
 
-    def IncludeFiles(
+    def include_files(
         self,
-        source_path: str,
-        target_path: str,
+        source_path: Union[Path, str],
+        target_path: Union[Path, str],
         copy_dependent_files: bool = True,
     ) -> None:
         """Include the files in the given directory in the target build."""
-        self.include_files.append((source_path, target_path))
+        self.included_files.extend(
+            process_path_specs([(source_path, target_path)])
+        )
         if not copy_dependent_files:
-            self.ExcludeDependentFiles(source_path)
+            self.exclude_dependent_files(source_path)
 
-    def IncludeModule(self, name: str) -> Module:
+    def include_module(self, name: str) -> Module:
         """Include the named module in the frozen executable."""
+        # includes has priority over excludes
+        if name in self.excludes and name in self._modules:
+            if self._modules[name] is None:
+                self.excludes.pop(name)
+                self._modules.pop(name)
+        # include module
         deferred_imports: DeferredList = []
         module = self._import_module(name, deferred_imports)
         self._import_deferred_imports(deferred_imports, skip_in_import=True)
         return module
 
-    def IncludePackage(self, name: str) -> Module:
-        """
-        Include the named package and any submodules in the frozen executable.
-        """
+    def include_package(self, name: str) -> Module:
+        """Include the named package and any submodules in the frozen
+        executable."""
         deferred_imports: DeferredList = []
         module = self._import_module(name, deferred_imports)
         if module.path:
@@ -650,7 +762,7 @@ class ModuleFinder:
         self._import_deferred_imports(deferred_imports, skip_in_import=True)
         return module
 
-    def ReportMissingModules(self) -> None:
+    def report_missing_modules(self) -> None:
         """Display a list of modules that weren't found."""
         if self._bad_modules:
             print("Missing modules:")
@@ -659,22 +771,30 @@ class ModuleFinder:
             for name in names:
                 callers = list(self._bad_modules[name].keys())
                 callers.sort()
-                print("? {} imported from {}".format(name, ", ".join(callers)))
-            print(
-                "This is not necessarily a problem - the modules "
-                "may not be needed on this platform.\n"
-            )
+                print(f"? {name} imported from", ", ".join(callers))
+            print("This is not necessarily a problem - the modules ", end="")
+            print("may not be needed on this platform.\n")
 
-    def SetOptimizeFlag(self, optimize_flag):
-        """Set a new value of optimize flag and returns the previous value."""
-        previous = self.optimize_flag
-        # The value of optimize_flag is propagated according to the user's
-        # choice and checked in dist.py or main,py. This value is unlikely
-        # to be wrong, yet we check and ignore any divergent value.
-        if -1 <= optimize_flag <= 2:
-            self.optimize_flag = optimize_flag
-        return previous
+    @property
+    def optimize_flag(self) -> int:
+        """The value of optimize flag propagated according to the user's
+        choice."""
+        return self._optimize_flag
 
-    def ZipIncludeFiles(self, source_path, target_path):
-        """Include the file(s) in the library.zip"""
-        self.zip_includes.append((source_path, target_path))
+    @optimize_flag.setter
+    def optimize_flag(self, value: int):
+        # The value of optimize_flag is checked in dist.py or main,py. This
+        # value is unlikely to be wrong, yet we check and ignore any divergent
+        # value.
+        if -1 <= value <= 2:
+            self._optimize_flag = value
+
+    def zip_include_files(
+        self,
+        source_path: Union[str, Path],
+        target_path: Optional[Union[str, Path, PurePath]] = None,
+    ) -> None:
+        """Include files or all of the files in a directory to the zip file."""
+        self.zip_includes.extend(
+            process_path_specs([(source_path, target_path)])
+        )
