@@ -22,6 +22,8 @@ STRINGREPLACE = list(
 
 
 class ExtensionFinder(PathFinder):
+    """A Finder for extension modules of packages in zip files."""
+
     @classmethod
     def find_spec(cls, fullname, path=None, target=None):
         """
@@ -46,6 +48,8 @@ class ExtensionFinder(PathFinder):
 
 
 def init():
+    """Basic initialization of the startup script."""
+
     # update sys module
     if sys.platform == "win32":
         # for MSYS2
@@ -53,6 +57,7 @@ def init():
         sys.executable = os.path.normpath(sys.executable)
     sys.frozen_dir = frozen_dir = os.path.dirname(sys.executable)
     sys.meta_path.append(ExtensionFinder)
+
     if sys.platform == "win32":
         # fix PATH for conda managers and MSYS2
         add_to_path = os.path.join(frozen_dir, "lib")
@@ -82,6 +87,8 @@ def init():
 
 
 def run():
+    """Determines the name of the initscript and execute it."""
+
     # get the real name of __init__ script
     # basically, the basename of executable plus __init__
     # but can be renamed when only one executable exists
@@ -90,14 +97,14 @@ def run():
         name, _ = os.path.splitext(name)
     name = name.partition(".")[0]
     if not name.isidentifier():
-        for ch in STRINGREPLACE:
-            name = name.replace(ch, "_")
+        for char in STRINGREPLACE:
+            name = name.replace(char, "_")
     name = os.path.normcase(name)
     try:
         module_init = __import__(name + "__init__")
     except ModuleNotFoundError:
         files = []
-        for k in __loader__._files:
+        for k in __loader__._files:  # pylint: disable=W0212
             if k.endswith("__init__.pyc"):
                 k = k.rpartition("__init__")[0]
                 if k.isidentifier():
